package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.PageResult;
import com.nuliji.tools.Transform;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.qxgmat.data.constants.enums.QuestionType;
import com.qxgmat.data.constants.enums.module.QuestionModule;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.dao.UserNoteMapper;
import com.qxgmat.data.dao.entity.Question;
import com.qxgmat.data.dao.entity.QuestionNo;
import com.qxgmat.data.dao.entity.UserCollectQuestion;
import com.qxgmat.data.dao.entity.UserNote;
import com.qxgmat.data.relation.UserNoteRelationMapper;
import com.qxgmat.data.relation.entity.UserNoteRelation;
import com.qxgmat.service.inline.QuestionNoService;
import com.qxgmat.service.inline.QuestionService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.List;

@Service
public class UserNoteService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(UserNoteService.class);

    @Resource
    private UserNoteMapper userNoteMapper;

    @Resource
    private UserNoteRelationMapper userNoteRelationMapper;

    @Resource
    private QuestionNoService questionNoService;

    @Resource
    private QuestionService questionService;

    /**
     * 获取用户笔记列表
     * @param page
     * @param size
     * @param userId
     * @param module
     * @param type
     * @param startTime
     * @param endTime
     * @param order
     * @param direction
     * @return
     */
    public PageResult<UserNoteRelation> list(int page, int size, Integer userId, QuestionModule module, QuestionType type, String startTime, String endTime, String order, DirectionStatus direction){
        String moduleKey = module == null ? null : module.key;
        String typeKey = type == null ? null : type.key;
        Page<UserNote> p = page(()->{
            userNoteRelationMapper.list(userId, moduleKey, typeKey, startTime, endTime, order, direction.key);
        }, page, size);

        Collection ids = Transform.getIds(p, UserNote.class, "id");

        // 获取详细数据
        List<UserNote> list = select(ids);

        List<UserNoteRelation> pr = Transform.convert(list, UserNoteRelation.class);

        // 获取题目信息
        Collection questionIds = Transform.getIds(pr, UserNoteRelation.class, "questionId");
        List<Question> questionList = questionService.select(questionIds);
        Transform.combine(pr, questionList, UserNoteRelation.class, "questionId", "question", Question.class, "id");

        // 获取题目编号信息
        Collection questionNoIds = Transform.getIds(pr, UserNoteRelation.class, "questionNoId");
        List<QuestionNo> questionNoList = questionNoService.select(questionNoIds);
        Transform.combine(pr, questionNoList, UserNoteRelation.class, "questionNoId", "questionNo", Question.class, "id");

        return new PageResult<>(pr, p.getTotal());
    }

    /**
     * 更新用户笔记：没有则添加
     * @param note
     * @return
     */
    @Transactional
    public UserNote update(UserNote note){
        Example example = new Example(UserCollectQuestion.class);
        example.and(
                example.createCriteria()
                        .andEqualTo("userId", note.getUserId())
        );
        example.and(
                example.createCriteria()
                        .orEqualTo("questionId", note.getQuestionId())
                        .orEqualTo("questionNoId", note.getQuestionNoId())
        );
        UserNote in = one(userNoteMapper, example);
        if(in == null){
            return add(note);
        }else{
            note.setId(in.getId());
            return edit(note);
        }
    }

    public UserNote add(UserNote note){
        int result = insert(userNoteMapper, note);
        note = one(userNoteMapper, note.getId());
        if(note == null){
            throw new SystemException("笔记添加失败");
        }
        return note;
    }

    public UserNote edit(UserNote note){
        UserNote in = one(userNoteMapper, note.getId());
        if(in == null){
            throw new ParameterException("笔记不存在");
        }
        int result = update(userNoteMapper, note);
        return note;
    }

    public boolean delete(Number id){
        UserNote in = one(userNoteMapper, id);
        if(in == null){
            throw new ParameterException("笔记不存在");
        }
        int result = delete(userNoteMapper, id);
        return result > 0;
    }

    public UserNote get(Number id){
        UserNote in = one(userNoteMapper, id);

        if(in == null){
            throw new ParameterException("笔记不存在");
        }
        return in;
    }

    public Page<UserNote> select(int page, int pageSize){
        return select(userNoteMapper, page, pageSize);
    }

    public List<UserNote> select(Collection ids){
        return select(userNoteMapper, ids);
    }

}
