package com.qxgmat.service;

import com.github.pagehelper.Page;
import com.nuliji.tools.AbstractService;
import com.nuliji.tools.PageResult;
import com.nuliji.tools.exception.ParameterException;
import com.nuliji.tools.exception.SystemException;
import com.nuliji.tools.mybatis.Example;
import com.qxgmat.data.constants.enums.QuestionType;
import com.qxgmat.data.constants.enums.module.PaperModule;
import com.qxgmat.data.constants.enums.status.DirectionStatus;
import com.qxgmat.data.dao.UserPaperMapper;
import com.qxgmat.data.dao.entity.UserPaper;
import com.qxgmat.data.relation.UserPaperRelationMapper;
import com.qxgmat.util.annotation.Callback;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class UserPaperService extends AbstractService {
    private static final Logger logger = LoggerFactory.getLogger(UserPaperService.class);
    protected boolean SOFT_FLAG = true;

    @Resource
    private UserPaperMapper userPaperMapper;

    @Resource
    private UserPaperRelationMapper userPaperRelationMapper;

    @Resource
    private ExercisePaperService exercisePaperService;

    @Resource
    private HomeworkPreviewService homeworkPreviewService;

    @Resource
    private ExaminationPaperService examinationPaperService;

    private Map<PaperModule, Callback> finishCallback;

    public UserPaperService(){
        finishCallback = new HashMap<>();
        finishCallback.put(PaperModule.EXERCISE, (obj)->{
            return exercisePaperService.finished((UserPaper) obj);
        });
        finishCallback.put(PaperModule.HOMEWORK_PREVIEW, (obj)->{
            return homeworkPreviewService.finished((UserPaper) obj);
        });
        finishCallback.put(PaperModule.EXAMINATION, (obj)->{
            return homeworkPreviewService.finished((UserPaper) obj);
        });
    }

    public PageResult<UserPaper> list(int page, int size, Integer userId, PaperModule module, QuestionType type, String startTime, String endTime, String order, DirectionStatus direction){

        return new PageResult<>(null, 0);
    }

    public Boolean removeByUsersAndPaper(Collection<Integer> userIds, PaperModule module, Integer paperId){
        if(userIds.size() == 0) return false;
        Example example = new Example(UserPaper.class);
        example.and(
                example.createCriteria()
                        .andIn("userId", userIds)
                        .andEqualTo("module", module.key)
                        .andEqualTo("moduleId", paperId)
        );
        return delete(userPaperMapper, example, SOFT_FLAG) > 0;
    }

    public Boolean editByUsersAndPaper(UserPaper userPaper, Collection<Integer> userIds, PaperModule module, Integer paperId){
        if(userIds.size() == 0) return false;
        Example example = new Example(UserPaper.class);
        example.and(
                example.createCriteria()
                        .andIn("userId", userIds)
                        .andEqualTo("module", module.key)
                        .andEqualTo("moduleId", paperId)
        );
        return update(userPaperMapper, example, userPaper) > 0;
    }

    public UserPaper add(UserPaper paper){
        int result = insert(userPaperMapper, paper);
        paper = one(userPaperMapper, paper.getId());
        if(paper == null){
            throw new SystemException("组卷添加失败");
        }
        return paper;
    }

    public UserPaper edit(UserPaper paper){
        UserPaper in = one(userPaperMapper, paper.getId());
        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        int result = update(userPaperMapper, paper);
        return paper;
    }

    public boolean delete(Number id){
        UserPaper in = one(userPaperMapper, id);
        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        int result = delete(userPaperMapper, id);
        return result > 0;
    }

    public UserPaper get(Number id){
        UserPaper in = one(userPaperMapper, id);

        if(in == null){
            throw new ParameterException("组卷不存在");
        }
        return in;
    }

    public Page<UserPaper> select(int page, int pageSize){
        return select(userPaperMapper, page, pageSize);
    }

    public List<UserPaper> select(Collection ids){
        return select(userPaperMapper, ids);
    }

}
